<?php
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use Ifsnop\Mysqldump as IMysqldump;

require_once ('config.php');
require_once ('vendor/autoload.php');


// Creates a mysqldump and emails the resulting dump file

// Edit the following values
$db_host	= DB_HOST; // usually localhost
$db_user 	= DB_USER;
$db_pass 	= DB_PASS;
$db_name 	= DB_NAME;
$eto 		= "benny.donny1@gmail.com";

// Remember to set the location of your backups dir and to set perms to 777
$backup_dir = dirname(__FILE__) . '/_dbbackups';

if (!file_exists($backup_dir)) {
	mkdir($backup_dir, 0777);
	//chmod($backup_dir, 0777);
}
if ($db_name == '') {
	$db_name = '--all-databases';
	$db_backup_filename = 'all_databases';
} else {
	$db_backup_filename = $db_name;
}
$backup_file = $backup_dir . '/' . $db_name . '_' . date("Y-m-d-H-i-s") . '.sql.gz'; // .gz	.bz2

// =================================================================================

try {
	$dumpSettings = array(
		'include-tables' => array(),
		'exclude-tables' => array(),
		'compress' => IMysqldump\Mysqldump::GZIP, //IMysqldump\Mysqldump::GZIP, IMysqldump\Mysqldump::BZIP2 or IMysqldump\Mysqldump::NONE
		'init_commands' => array(),
		'where' => ''
	);
	$pdoSettings = array(
		PDO::ATTR_PERSISTENT => true,
		PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
		PDO::MYSQL_ATTR_USE_BUFFERED_QUERY => true // for large data, set MYSQL_ATTR_USE_BUFFERED_QUERY to false
	);

	$dump = new IMysqldump\Mysqldump("mysql:host=$db_host;dbname=$db_name", $db_user, $db_pass, $dumpSettings, $pdoSettings);
	$dump->start($backup_file);
	
	
	// Mail the file

	$mailer             = new PHPMailer;
	$mailer->Host       = $config['smtp_host']; 
	$mailer->Username   = $config['smtp_username'];
	$mailer->Password   = $config['smtp_password'];
	$mailer->Port       = $config['smtp_port'];
	$mailer->SMTPAuth   = $config['smtp_auth'];
	$mailer->SMTPSecure = $config['smtp_secure'];
	$mailer->SMTPDebug  = false;
	$mailer->IsSMTP();                 
	$mailer->From       = $config['smtp_from'];
	$mailer->FromName   = "Auto MySQL Backup";            
	$mailer->AddAddress($eto);
	$mailer->Subject = "Auto MySQL Backup Triggered ::: " . $config['app_domain'];
	$body = "Here is your backup for {$config['app_name']} => {$config['app_domain']}";
	$mailer->Body = nl2br($body);
	$mailer->AltBody = $body;
	$mailer->AddAttachment($backup_file);
	$mailer->WordWrap = 50; // Some old mail programs cut off email if word wrap is greater

	// Send email using smtp
	$mailer->Send();
	
} catch (\Exception $e) {
	echo 'mysqldump-php error: ' . $e->getMessage();
}



// =========== DELETE OLD BACKUP FILES, LEAVE THE NEWEST 3 BACKUPS
$keep_newest = 3;
// $files = glob($backup_dir . '/*', GLOB_ONLYDIR);
$files = glob($backup_dir . '/*'); 
// sort by time last modified
usort($files, function($a, $b) {
    return filemtime($a) - filemtime($b);
});
// exclude the newest numbers from the array list to be deleted
$deletions = array_slice($files, 0, count($files) - $keep_newest);
// loop through and delete
foreach($deletions as $to_delete) {
	chmod($to_delete, 0777); // change permission mode
	
	if(is_dir($to_delete)) { 
		array_map('unlink', glob("$to_delete/*")); // *.*
		$deleted = rmdir($to_delete);
	} else {
		unlink($to_delete);
	}    
}
clearstatcache();
// =========== End of Deletion
