<?php  
/**
 * @package SMS API Library (Class) for Nimbow.com API
*/
class SMS_Lib {
    
    /**
     * API Key    
     */
	private $key = '';
    
    
    /**
     * API Command execution via ('curl' or 'fopen') 
     */
    private $execute_api_commands_via = 'curl'; //'fopen' or 'curl'
    
    /**
     * Submit data API via ('post' or 'get') 
     */
    private $submit_api_data_via = 'get'; //'post' or 'get'
	
    
    /**
     *  Class constructor
     * 
     * @param string $key
     */
    public function __construct($key, $submit_api_data_via = 'post')
    {
        $this->key =  $key;       
        $this->submit_api_data_via = $submit_api_data_via;    
    }
    
    
	/**
     * Send a message. 
     * 
     * @param string $sender: Sender (Maximum of 11 characters or a phone number). The FROM part of the SMS can be alphanumeric or just numbers
     * @param string $send_to: The destination phone numbers in international format without + or space e.g: 2348057055555
     * @param string $message: The SMS Message you wish to send. Maximum of 160 characters for single message.
     * @return array MESSAGE RESULTS -or- ERROR DESCRIPTION 
     */
	public function send_message($sender, $send_to, $message)
	{
       
	   if (!empty($this->key)) {
	       
           $api_url =  'http://api.nimbow.com/sms'; // without the ? of querystring
            
            //GET request
            if ($this->submit_api_data_via == 'get') {                
                
                /** create the required GET URL */       
                //e.g // http://api.nimbow.com/sms?key=COdk59Q1dmr&from=Max&to=4917212345670&text=Hi                
                $url_for_get_method = $api_url . '?'
                                    . 'key=' . urlencode($this->key)
                                    . '&from=' . urlencode($sender)
                                    . '&to=' . urlencode($send_to)
                                    . '&text=' . urlencode($message)
                                    . '&GetMessageId=1'
                                    . '&GetFrom=1'
                                    . '&GetTo=1'
                                    . '&GetCost=1'
                                    . '&GetDeliveryReport=1';
                //execute the url via GET
                $response = $this->_execute_api_commands_via_get($url_for_get_method);
                
            //POST Request  
            } else {
    
                /** create data array to submit via POST method */                         
                $data_for_post_method =  array(
                                        'key' => $this->key,
                                        'from' => $sender,
                                        'to' => $send_to,
                                        'text' => $message,
                                        'GetMessageId' => 1,
                                        'GetFrom' => 1,
                                        'GetTo' => 1,
                                        'GetCost' => 1,
                                        'GetDeliveryReport' => 1
                );               
                //submit the data via POST
                $response = $this->_execute_api_commands_via_post($api_url, $data_for_post_method);
            }       
            
            $response = json_decode($response, true); 
            $response_info = $response;
            
            if ($response_info['StatusCode'] === 0) {
                $data['status']             = 'OK';
                $data['message']            = "Message sent.";
                $data['status_code']        = $response_info['StatusCode'];
                $data['to']                 = $response_info['To'];
                $data['from']                 = $response_info['From'];
                $data['message_id']         = $response_info['MessageId'];
                //$data['message_price']      = $response_info['Cost'];
            } else {
                $data['status']             = 'ERR';
                $data['status_code']        = $response_info['StatusCode'];
                
                if ($response_info['StatusCode'] == 1) {
                    $data['message']            = "Internal Error: An error has occurred in the platform while processing this message.";
                } elseif ($response_info['StatusCode'] == 2) {
                    $data['message']            = "Unknown Error: An undocumented error occurred.";
                } elseif ($response_info['StatusCode'] == 3) {
                    $data['message']            = "Out of Credits: Your account does not have sufficient fund to process this message.";
                } elseif ($response_info['StatusCode'] == 4) {
                    $data['message']            = "Invalid API Call: A valid URL to call the API must be used.";
                } elseif ($response_info['StatusCode'] == 5) {
                    $data['message']            = "Invalid API Credentials:	The provided API key could be wrong or the IP Addressed used is not configured to be allowed to use the API.";
                } elseif ($response_info['StatusCode'] == 101) {
                    $data['message']            = "to  missing: The request contains no receiver information.";
                } elseif ($response_info['StatusCode'] == 102) {
                    $data['message']            = "text  missing or empty	The request contains no message content.";
                } elseif ($response_info['StatusCode'] == 200) {
                    $data['message']            = "from  invalid length: The numeric sender is allowed to contain max. 16 digits.";
                } elseif ($response_info['StatusCode'] == 204) {
                    $data['message']            = "from  invalid alpha numeric length: The alpha numeric sender is allowed to have 1 to 11 characters.";
                } else {
                    $data['message']            = "Unknown Error Code: An undocumented error occurred.";
                }
 
            }                                   
            return $data;            
                           
        } else {
            
            $data['status'] = 'ERR';
            $data['message'] = "Please specify your API Key and API Secret.";
            return $data;
            
        }//end if (! empty($this->session_id)) {
    }//end of function send_message()
 
      
    
    /**
     * Do backend execution of the API commands via GET method with fopen or CURL
     * This should not be called directly but from a member function of this class
     * 
     * @param array The post data ... this can be form data e.g array('sender'=>'Alert', 'to'=> '23480564333')...
     * @return string Delivery status or Error Status
     */
    private function _execute_api_commands_via_post($api_url, $post_data)
	{
	   if (($this->execute_api_commands_via == 'curl') &&  (function_exists('curl_init') == TRUE)){
	       
           // create a new cURL resource
            $resource_ch = curl_init();           
            // set appropriate options
            curl_setopt($resource_ch, CURLOPT_URL, $api_url);
            curl_setopt($resource_ch, CURLOPT_HEADER, FALSE);
            curl_setopt($resource_ch, CURLOPT_RETURNTRANSFER, TRUE); //allow it to return transfer response
            curl_setopt($resource_ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($resource_ch, CURLOPT_POST, TRUE); //send via HTTP POST Method
            curl_setopt($resource_ch, CURLOPT_POSTFIELDS, $post_data); //Array of data to send            
            // grab URL and pass it to the browser and get the return transfer
            $response = curl_exec($resource_ch);           
            // close cURL resource, and free up system resources
            curl_close($resource_ch);
            
            return $data = $response;
	       
	   } elseif (($this->execute_api_commands_via == 'fopen') &&  (ini_get('allow_url_fopen') == TRUE)) {
           
            $data['status'] = 'ERR';
            $data['message'] = "You can only send GET method data via 'fopen', please use cURL (Client URL) Library to send POST method data.";
            return $data;
	       
	   } else {
	       
	       $data['status'] = 'ERR';
           $data['message'] = "Please enable 'allow_url_fopen' or cURL (Client URL) library";
	       return $data;
	   } 
    }//end of function _execute_api_commands_via_get()

    
    /**
     * Do backend execution of the API commands via GET method with fopen or CURL
     * This should not be called directly but from a member function of this class
     * 
     * @param string URL
     * @return string Delivery status or Error Status
     */
    private function _execute_api_commands_via_get($url)
	{
	   if (($this->execute_api_commands_via == 'curl') &&  (function_exists('curl_init') == TRUE)){
	       
           // create a new cURL resource
            $resource_ch = curl_init();           
            // set appropriate options
            curl_setopt($resource_ch, CURLOPT_URL, $url);
            curl_setopt($resource_ch, CURLOPT_HEADER, FALSE);
            curl_setopt($resource_ch, CURLOPT_RETURNTRANSFER, TRUE); //allow it to return transfer response
            curl_setopt($resource_ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            // grab URL and pass it to the browser and get the return transfer
            $response = curl_exec($resource_ch);           
            // close cURL resource, and free up system resources
            curl_close($resource_ch);
            
            return $data = $response;
	       
	   } elseif (($this->execute_api_commands_via == 'fopen') &&  (ini_get('allow_url_fopen') == TRUE)) {
	       
           /* call the URL */
           if ($fopen_handle = @fopen($url, "r")) { 
                $response = fgets($fopen_handle, 255);
                return $data = $response;
                fclose($fopen_handle);
                
            } else {
                
                $data['status'] = 'ERR';
                $data['message'] = "Error: URL could not be opened.";
                return $data;
            }
	       
	   } else {
	       
	       $data['status'] = 'ERR';
           $data['message'] = "Please enable 'allow_url_fopen' or cURL (Client URL) library";
	       return $data;
	   } 
    }//end of function _execute_api_commands_via_get()
    
    
} // end of class