<?php  
/**
 * @package SMS API Library (Class) for Nexmo.com API
*/
class SMS_Lib {
    
    /**
     * API Key    
     */
	private $api_key = '';
    
    /**
     * API Secret     
     */
	private $api_secret = '';
    
    
    /**
     * API Command execution via ('curl' or 'fopen') 
     */
    private $execute_api_commands_via = 'curl'; //'fopen' or 'curl'
    
    /**
     * Submit data API via ('post' or 'get') 
     */
    private $submit_api_data_via = 'get'; //'post' or 'get'
	
    
    /**
     *  Class constructor
     * 
     * @param string $api_key
     * @param string $api_secret
     */
    public function __construct($api_key, $api_secret, $submit_api_data_via = 'post')
    {
        $this->api_key =  $api_key;       
        $this->api_secret =  $api_secret;   
        $this->submit_api_data_via = $submit_api_data_via;    
    }
    
    
	/**
     * Send a message. 
     * 
     * @param string $sender: Sender (Maximum of 11 characters or a phone number). The FROM part of the SMS can be alphanumeric or just numbers
     * @param string $send_to: The destination phone numbers in international format without + or space e.g: 2348057055555
     * @param string $message: The SMS Message you wish to send. Maximum of 160 characters for single message.
     * @return array MESSAGE RESULTS -or- ERROR DESCRIPTION 
     */
	public function send_message($sender, $send_to, $message)
	{
       
	   if (!empty($this->api_key)) {
	       
           $api_url =  'https://renexhost.com/retsms-api/post.php'; // without the ? of querystring
            
            //GET request
            if ($this->submit_api_data_via == 'get') {                
                
                /** create the required GET URL */       
                //e.g // https://retenvo.com/retsms-api/post.php?command=send-sms&api_key=xxx&from=xxx&to=xxx&message=xxx                
                $url_for_get_method = $api_url . '?'
                                    . 'command=send-sms'
                                    . 'api_key=' . urlencode($this->api_key)
                                    . '&from=' . urlencode($sender)
                                    . '&to=' . urlencode($send_to)
                                    . '&message=' . urlencode($message);
                //execute the url via GET
                $response = $this->_execute_api_commands_via_get($url_for_get_method);
                
            //POST Request  
            } else {
    
                /** create data array to submit via POST method */                         
                $data_for_post_method =  array(
                                        'command' => 'send-sms',
                                        'api_key' => $this->api_key,
                                        'from' => $sender,
                                        'to' => $send_to,
                                        'message' => $message
                );               
                //submit the data via POST
                $response = $this->_execute_api_commands_via_post($api_url, $data_for_post_method);
            }       
            
            $response = json_decode($response, true); 
            
            if ($response['status'] == true) {
                $data['status']             = 'OK';
                $data['message']            = $response['message'];
                $data['data']            	= $response['data'];
            } else {
                $data['status']             = 'ERR';
                $data['message']            = $response['message'];
                $data['data']        		= $response['data'];
            }                                   
            return $data;            
                           
        } else {
            
            $data['status'] = 'ERR';
            $data['message'] = "Please specify your API Key and API Secret.";
            return $data;
            
        }//end if (! empty($this->session_id)) {
    }//end of function send_message()
 
      
    
    /**
     * Do backend execution of the API commands via GET method with fopen or CURL
     * This should not be called directly but from a member function of this class
     * 
     * @param array The post data ... this can be form data e.g array('sender'=>'Alert', 'to'=> '23480564333')...
     * @return string Delivery status or Error Status
     */
    private function _execute_api_commands_via_post($api_url, $post_data)
	{
	   if (($this->execute_api_commands_via == 'curl') &&  (function_exists('curl_init') == TRUE)){
	       
           // create a new cURL resource
            $resource_ch = curl_init();           
            // set appropriate options
            curl_setopt($resource_ch, CURLOPT_URL, $api_url);
            curl_setopt($resource_ch, CURLOPT_HEADER, FALSE);
            curl_setopt($resource_ch, CURLOPT_RETURNTRANSFER, TRUE); //allow it to return transfer response
            curl_setopt($resource_ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($resource_ch, CURLOPT_POST, TRUE); //send via HTTP POST Method
            curl_setopt($resource_ch, CURLOPT_POSTFIELDS, $post_data); //Array of data to send            
            // grab URL and pass it to the browser and get the return transfer
            $response = curl_exec($resource_ch);           
            // close cURL resource, and free up system resources
            curl_close($resource_ch);
            
            return $data = $response;
	       
	   } elseif (($this->execute_api_commands_via == 'fopen') &&  (ini_get('allow_url_fopen') == TRUE)) {
           
            $data['status'] = 'ERR';
            $data['message'] = "You can only send GET method data via 'fopen', please use cURL (Client URL) Library to send POST method data.";
            return $data;
	       
	   } else {
	       
	       $data['status'] = 'ERR';
           $data['message'] = "Please enable 'allow_url_fopen' or cURL (Client URL) library";
	       return $data;
	   } 
    }//end of function _execute_api_commands_via_get()

    
    /**
     * Do backend execution of the API commands via GET method with fopen or CURL
     * This should not be called directly but from a member function of this class
     * 
     * @param string URL
     * @return string Delivery status or Error Status
     */
    private function _execute_api_commands_via_get($url)
	{
	   if (($this->execute_api_commands_via == 'curl') &&  (function_exists('curl_init') == TRUE)){
	       
           // create a new cURL resource
            $resource_ch = curl_init();           
            // set appropriate options
            curl_setopt($resource_ch, CURLOPT_URL, $url);
            curl_setopt($resource_ch, CURLOPT_HEADER, FALSE);
            curl_setopt($resource_ch, CURLOPT_RETURNTRANSFER, TRUE); //allow it to return transfer response
            curl_setopt($resource_ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            // grab URL and pass it to the browser and get the return transfer
            $response = curl_exec($resource_ch);           
            // close cURL resource, and free up system resources
            curl_close($resource_ch);
            
            return $data = $response;
	       
	   } elseif (($this->execute_api_commands_via == 'fopen') &&  (ini_get('allow_url_fopen') == TRUE)) {
	       
           /* call the URL */
           if ($fopen_handle = @fopen($url, "r")) { 
                $response = fgets($fopen_handle, 255);
                return $data = $response;
                fclose($fopen_handle);
                
            } else {
                
                $data['status'] = 'ERR';
                $data['message'] = "Error: URL could not be opened.";
                return $data;
            }
	       
	   } else {
	       
	       $data['status'] = 'ERR';
           $data['message'] = "Please enable 'allow_url_fopen' or cURL (Client URL) library";
	       return $data;
	   } 
    }//end of function _execute_api_commands_via_get()
    
    
} // end of class