<?php
/**
 * Transaction Class
 */
class Transaction {
    
    private $Database;
    
    
    public function __construct()
    {
        $this->Database = new Database();
    }
    
    public function add_cargo($user_id, $tracking_code, $description, $sender, $receiver, $receiver_phone, $receiver_email, $origin, $destination, $status, $date_time, $exp_datetime=null)
    {        
        $user_id        = trim($user_id);
        $tracking_code  = trim($tracking_code);
        $description    = trim($description);
        $sender         = trim($sender);
        $receiver       = trim($receiver);
        $receiver_phone = trim($receiver_phone);
        $receiver_email = trim($receiver_email);
        $origin         = trim($origin);
        $destination    = trim($destination);
        $status         = trim($status);
        $date_time      = trim($date_time);
        //$date_time      = date('Y-m-d G:i:s');
		$exp_datetime 	= (!empty($exp_datetime)) ? trim($exp_datetime) : null;
        

        $this->Database->prepare_query("INSERT INTO `cargo` (`user_id`, `tracking_code`, `description`, `sender`, `receiver`, `receiver_phone`, `receiver_email`, `origin`, `destination`, `status`, `date_added`, `expected_delivery_date`) VALUES (:uid, :tc, :desc, :sen, :rec, :rec_ph, :rec_em, :orig, :dest, :stat, :dt, :exp_dt)");
        $this->Database->bind_value(':uid', $user_id);
        $this->Database->bind_value(':tc', $tracking_code);
        $this->Database->bind_value(':desc', $description);
        $this->Database->bind_value(':sen', $sender);
        $this->Database->bind_value(':rec', $receiver);
        $this->Database->bind_value(':rec_ph', $receiver_phone);
        $this->Database->bind_value(':rec_em', $receiver_email);
        $this->Database->bind_value(':orig', $origin);
        $this->Database->bind_value(':dest', $destination);
        $this->Database->bind_value(':stat', $status);
        $this->Database->bind_value(':dt', $date_time);
		$this->Database->bind_value(':exp_dt', $exp_datetime);

        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return (int) $this->Database->last_insert_id(); 
        } else {
            return false;
        }              
    }
    
    public function track_cargo($tracking_code)
    {        
        $tracking_code  = trim($tracking_code);
        
        $this->Database->prepare_query("SELECT * FROM `cargo` WHERE `tracking_code`=:tc LIMIT 1");
        $this->Database->bind_value(':tc', $tracking_code);     
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return $this->Database->fetch_single_result(); 
        } else {
            return false;
        }              
    }
    
    public function get_cargo($cargo_id)
    {        
        $cargo_id  = trim($cargo_id);
        
        $this->Database->prepare_query("SELECT * FROM `cargo` WHERE `cargo_id`=:cid LIMIT 1");
        $this->Database->bind_value(':cid', $cargo_id);     
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return $this->Database->fetch_single_result(); 
        } else {
            return false;
        }              
    }
    
    public function get_cargoes($filter_user_id = null)
    {        
        if (!empty($filter_user_id)) {
            $this->Database->prepare_query("SELECT * FROM `cargo` WHERE `user_id`=:uid ORDER BY `cargo_id` DESC");
            $this->Database->bind_value(':uid', $filter_user_id);
        } else {
            $this->Database->prepare_query("SELECT * FROM `cargo` ORDER BY `cargo_id` DESC");
        }       
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return $this->Database->fetch_all_result(); 
        } else {
            return false;
        }              
    }
    
    public function update_cargo_field($cargo_id, $field_name, $field_value) 
    {
        $cargo_id       =  trim($cargo_id);
        $field_name     =  trim($field_name);
        $field_value    =  (!empty($field_value)) ? trim($field_value) : null;
        $this->Database->prepare_query("UPDATE `cargo` SET `$field_name`= :fieldvalue WHERE `cargo_id`= :cid");
        $this->Database->bind_value(':cid', $cargo_id);
        $this->Database->bind_value(':fieldvalue', $field_value);
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return true; 
        } else {
            return false;
        }         
    }
    
    public function delete_cargo($cargo_id) 
    {
        $cargo_id     =  trim($cargo_id);
        $this->Database->prepare_query("DELETE c.*, c_h.* FROM `cargo` AS c LEFT JOIN `cargo_history` AS c_h  ON c.`cargo_id`=c_h.`cargo_id` WHERE c.`cargo_id`= :cid");
        $this->Database->bind_value(':cid', $cargo_id);
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return true; 
        } else {
            return false;
        }         
    }
    

    
    public function add_cargo_history($cargo_id, $remark, $location, $date_time)
    {        
        $cargo_id   = trim($cargo_id);
        $remark     = trim($remark);
        $location   = trim($location);
        $date_time  = trim($date_time);
        //$date_time  = date('Y-m-d G:i:s');

        $this->Database->prepare_query("INSERT INTO `cargo_history` (`cargo_id`, `remark`, `location`, `date_added`) VALUES (:cid, :rem, :loc, :dt)");
        $this->Database->bind_value(':cid', $cargo_id);
        $this->Database->bind_value(':rem', $remark);
        $this->Database->bind_value(':loc', $location);
        $this->Database->bind_value(':dt', $date_time);

        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return true; 
        } else {
            return false;
        }              
    }
    
    public function get_cargo_history($history_id)
    {        
        $history_id  = trim($history_id);
        
        $this->Database->prepare_query("SELECT * FROM `cargo_history` WHERE `history_id`=:hid LIMIT 1");
        $this->Database->bind_value(':hid', $history_id);     
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return $this->Database->fetch_single_result(); 
        } else {
            return false;
        }              
    }
    
    public function get_cargo_histories($cargo_id)
    {        
        $cargo_id  = trim($cargo_id);
        
        $this->Database->prepare_query("SELECT * FROM `cargo_history` WHERE `cargo_id`=:cid ORDER BY `history_id` DESC");
        $this->Database->bind_value(':cid', $cargo_id);    
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return $this->Database->fetch_all_result(); 
        } else {
            return false;
        }              
    }
	
	public function get_last_cargo_history($cargo_id)
    {        
        $cargo_id  = trim($cargo_id);
        
        $this->Database->prepare_query("SELECT * FROM `cargo_history` WHERE `cargo_id`=:cid ORDER BY `history_id` DESC LIMIT 1");
        $this->Database->bind_value(':cid', $cargo_id);    
        $this->Database->execute_query(__METHOD__);              
		if ($this->Database->count_row() > 0) {
            return $this->Database->fetch_single_result(); 
        } else {
            return false;
        } 
    }
    
    public function update_cargo_history_field($history_id, $field_name, $field_value) 
    {
        $history_id     =  trim($history_id);
        $field_name     =  trim($field_name);
        $field_value    =  trim($field_value);
        $this->Database->prepare_query("UPDATE `cargo_history` SET `$field_name`= :fieldvalue WHERE `history_id`= :hid");
        $this->Database->bind_value(':hid', $history_id);
        $this->Database->bind_value(':fieldvalue', $field_value);
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return true; 
        } else {
            return false;
        }         
    }
    
    public function delete_cargo_history($history_id) 
    {
        $history_id     =  trim($history_id);
        $this->Database->prepare_query("DELETE FROM `cargo_history` WHERE `history_id`= :hid");
        $this->Database->bind_value(':hid', $history_id);
        $this->Database->execute_query(__METHOD__);
        if ($this->Database->count_row() > 0) {
            return true; 
        } else {
            return false;
        }         
    }

}
?>